# Author : Patiga
# License: MIT
# Rewrite of the libtw2 tool "ddnet_properties"

import argparse
import twmap
import numpy as np
import msgpack

tiles = {
    12: "DFREEZE",
    16: "WALLJUMP",
    17: "EHOOK_START",
    19: "HIT_START",
    21: "SOLO_START",
    96: "TELE_GUN",
    104:  "NPC_START",
    105:  "SUPER_START",
    106:  "JETPACK_START",
    107:  "NPH_START",
    112:  "TELE_GRENADE",
    128:  "TELE_LASER",
    199:  "WEAPON_SHOTGUN",
    200:  "WEAPON_GRENADE",
    201:  "POWERUP_NINJA",
    202:  "WEAPON_RIFLE",
}

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="DDNet map properties extractor")
    parser.add_argument("MAP", help="map file to use")
    parser.add_argument("OUTPUT", help="output file")
    args = parser.parse_args()

    map = twmap.Map(args.MAP)

    game_tiles = map.game_layer().tiles
    found_tile_ids = set(np.unique(game_tiles[:, :, 0]))

    if map.front_layer() is not None:
        front_tiles = map.front_layer().tiles
        found_tile_ids |= set(np.unique(front_tiles[:, :, 0]))
        
    with open(args.OUTPUT, "wb") as file:
        packer = msgpack.Packer()
        file.write(packer.pack(map.game_layer().width()))
        file.write(packer.pack(map.game_layer().height()))

        length = sum([tile_id in found_tile_ids for tile_id in tiles.keys()])
        file.write(packer.pack_map_header(int(length)))

        for tile_id in sorted(tiles):
            if tile_id in found_tile_ids:
                file.write(packer.pack(tiles[tile_id]))
                file.write(packer.pack(True))
